<?php

namespace Drupal\google_stt\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Google\Cloud\Speech\V1\RecognitionAudio;
use Google\Cloud\Speech\V1\RecognitionConfig;
use Google\Cloud\Speech\V1\RecognitionConfig\AudioEncoding;
use Google\Cloud\Speech\V1\SpeechClient;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

/**
 * Class SpeechToTextForm.
 */
class SpeechToTextForm extends FormBase {

  /**
   * Drupal\Core\File\FileSystemInterface definition.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a new SpeechToTextForm object.
   */
  public function __construct(
    FileSystemInterface $file_system,
    EntityTypeManagerInterface $entity_type_manager
  ) {
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('file_system'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'speech_to_text_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Check if Google Service Account credentials json file is present.
    if (!file_exists('sites/default/service-account.json')) {
      drupal_set_message($this->t('The Google API Credentials file is missingg. ' .
        'Go <a href="https://console.cloud.google.com/apis/credentials" target="_blank">here</a>' .
        ' and do that. More information <a href="https://cloud.google.com/speech-to-text/docs/quickstart" target="_blank">here</a> on how to do that.'), 'warning');
    }
    $form['language_code'] = [
      '#type' => 'select',
      '#title' => 'Language Code',
      '#options' => ['en-US', 'en-UK', 'en-IN'],
      '#description' => 'Choose the English language variant.',
      '#default_value' => 'en-US',
      '#required' => TRUE,
    ];

    $form['upload_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Upload File'),
      '#description' => $this->t('Upload the Audio/Video file to be transcribed. Currently supported file types: .wav, .flac, .m4a, .mp4, .mp3'),
      '#required' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['mp3 mp4 m4a flac wav mpeg ogg m4p m4v avi wmv mov'],
        'file_validate_size' => [1024 * 1024 * 1024],
      ],
      '#upload_location' => 'public://audio_files',
    ];

    $form['actions'] = [
      '#type' => 'button',
      '#value' => $this->t('Transcribe'),
      '#ajax' => [
        'callback' => '::transcribeFile',
        'wrapper' => 'transcribe-output',
        'progress' => [
          'type' => 'throbber',
          'message' => $this->t('Transcribing...'),
        ]
      ]
    ];
    $form['#suffix'] = '<div id="transcribe-output"></div>';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  public function transcribeFile(array &$form, FormStateInterface $form_state) {
    putenv("GOOGLE_APPLICATION_CREDENTIALS=sites/default/service-account.json");
    // Display result.
    $fid = reset($form_state->getValue('upload_file'));
    $file = $this->entityTypeManager->getStorage('file')->load($fid);
    $real_path = $this->fileSystem->realpath($file->getFileUri());
    # The audio file's encoding, sample rate and language
    $audioDeets = reset(reset(json_decode(shell_exec("ffprobe -v quiet -print_format json -show_streams -print_format json $real_path"), TRUE)));
    $encoding = ($audioDeets['codec_name'] == 'flac') ? AudioEncoding::FLAC : AudioEncoding::LINEAR16;
    $sample_rate_hertz = $audioDeets['sample_rate'];
    $codec_name = $audioDeets['codec_name'];
    if (in_array($codec_name, ['flac', 'wav'])) {
      if ($audioDeets['channels'] > 1) {
        $output_file = $this->fileSystem->realpath(file_default_scheme() . "://temp.$codec_name");
        shell_exec("ffmpeg -i $real_path -ac 1 $output_file");
      }
    }
    else {
      $output_file = $this->fileSystem->realpath(file_default_scheme() . "://temp.wav");
      $encoding = AudioEncoding::LINEAR16;
      shell_exec("ffmpeg -i $real_path -ac 1 $output_file");
      $audioDeets = reset(reset(json_decode(shell_exec("ffprobe -v quiet -print_format json -show_streams -print_format json $output_file"), TRUE)));
      $sample_rate_hertz = $audioDeets['sample_rate'];
    }
    $content = file_get_contents($output_file);
    # set string as audio content
    $audio = (new RecognitionAudio())
      ->setContent($content);
    $config = new RecognitionConfig([
      'encoding' => $encoding,
      'sample_rate_hertz' => $sample_rate_hertz,
      'language_code' => 'en-US'
    ]);
    # Instantiates a client
    $client = new SpeechClient();
    # Detects speech in the audio file
    $response = $client->recognize($config, $audio);
    # Print most likely transcription
    foreach ($response->getResults() as $result) {
      $alternatives = $result->getAlternatives();
      $mostLikely = $alternatives[0];
      $transcript = $mostLikely->getTranscript();
    }
    $client->close();
    shell_exec("rm $output_file");
    $response = new AjaxResponse();
    $response->addCommand(new HtmlCommand('#transcribe-output', '<h3>Transcript : </h3>' . $transcript));
    return $response;
  }
}
